/*
 * WiiMoteThread.cpp
 *
 *  Created on: Feb 15, 2011
 *      Author: maciej halasz
 */


#include <stdio.h>
#include <stdlib.h>

//#include <glib.h>
#include <cwiid.h>

#include "WiiMoteThread.h"

	cwiid_wiimote_t *wiimote;       /* wiimote handle */
	struct cwiid_state state;       /* wiimote state */
	int flipped;
	struct acc_cal wm_cal;
	WiiMoteThread* wiimoteclass;

/* pass an array of messages, all of which were received at the same time.
 * The id is to distinguish between multiple wiimotes using the same callback.
 * */
void cwiid_callback(cwiid_wiimote_t *wiimote, int mesg_count,
                    union cwiid_mesg mesg[], struct timespec *timestamp)
{
        int i, j, x, y, z;
        int valid_source;


        for (i=0; i < mesg_count; i++)
        {
                switch (mesg[i].type) {
                case CWIID_MESG_BTN:
                        printf("Button Report: %.4X\n", mesg[i].btn_mesg.buttons);
                        break;
                case CWIID_MESG_ACC:
                        //calibrate the reading
                        x = ((double)mesg[i].acc_mesg.acc[CWIID_X] - wm_cal.zero[CWIID_X]) /
                              (wm_cal.one[CWIID_X] - wm_cal.zero[CWIID_X]);
                        y = ((double)mesg[i].acc_mesg.acc[CWIID_Y] - wm_cal.zero[CWIID_Y]) /
                              (wm_cal.one[CWIID_Y] - wm_cal.zero[CWIID_Y]);
                        z = ((double)mesg[i].acc_mesg.acc[CWIID_Z] - wm_cal.zero[CWIID_Z]) /
                              (wm_cal.one[CWIID_Z] - wm_cal.zero[CWIID_Z]);

                        //printf("Acc Report: x=%d, y=%d, z=%d\n",x,y,z);

                        //check for gesture
                        if (flipped){
                          if (0.8 < z && z < 1.2){
                            flipped = 0;
                            wiimoteclass->sendSignalFlip(flipped);


                            printf("garage door ......opened\n");

                          }
                        }else{
                          if (-0.8 > z && z > -1.2 && -0.3 < y && y < 0.3){
                            flipped = 1;
                            wiimoteclass->sendSignalFlip(flipped);

                            printf("garage door ..... closed\n");
                          }
                        }
                        //update_garage_state();
                        break;
                case CWIID_MESG_ERROR:
                        if (cwiid_close(wiimote)) {
                                fprintf(stderr, "Error on wiimote disconnect\n");
                                exit(-1);
                        }
                        exit(0);
                        break;
                default:
                        printf("Other option");
                        break;
                }
        }
}


WiiMoteThread::WiiMoteThread() {
	// TODO Auto-generated constructor stub

	wiimoteclass = this;
}

WiiMoteThread::~WiiMoteThread() {
	// TODO Auto-generated destructor stub
}


void WiiMoteThread::run()
{

	pairing();
	exec();
}

void WiiMoteThread::pairing()
{
        bdaddr_t bdaddr;        /* bluetooth device address */
        int paired;

        //accept connections from any remotes
        bdaddr = *BDADDR_ANY;

        /* Connect to the wiimote */
        printf("Put Wiimote in discoverable mode now (press 1+2)...\n");
        paired=0;
        do {
                if (!(wiimote = cwiid_open(&bdaddr, CWIID_FLAG_MESG_IFC))) {
                        fprintf(stderr, "Unable to connect to wiimote - retrying\n");
                }else{
                        paired=1;
                }

        }while(!paired);

        // Register a callback function
        if (cwiid_set_mesg_callback(wiimote, cwiid_callback)) {
                fprintf(stderr, "Unable to set message callback\n");
        }

        //Turn on all LEDS on the remote
        cwiid_set_led(wiimote, CWIID_LED1_ON | CWIID_LED2_ON | CWIID_LED3_ON | CWIID_LED4_ON);

        //Vibrate the remote for 1 sec
        if (cwiid_set_rumble(wiimote, 1)) {
                fprintf(stderr, "Error setting rumble\n");
        }
        sleep(1);

        if (cwiid_set_rumble(wiimote, 0)) {
                fprintf(stderr, "Error setting rumble\n");
        }

        // calibration helps us interpret accelerometer readings...
        if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NONE, &wm_cal)) {
                printf("Calibration set! ");
        }

        flipped = 0;
        cwiid_set_rpt_mode(wiimote, CWIID_RPT_ACC);




}

void WiiMoteThread::sendSignalFlip(int state)
{

	emit flipGarageDoorSwitch(state);
}







